#!/usr/bin/Rscript

source("../lib/curve.r")
source("../lib/stats.r")
source("../lib/util.r")
source("../lib/eia.r")

# We have a special set of exclusions, because we don't want to show
# small-population countries that mess up the range of the plot.
exclude <- c("North America",
	     "Central & South America",
	     "Asia & Oceania",
	     "Europe",
	     "Eurasia",
	     "Africa",
	     "Middle East",
	     "Antarctica",
	     "Virgin Islands,  U.S.",
	     "Gibraltar",
	     "World")

pdf("Figure 3a.pdf")
par(mar=c(4.5, 4.5, 2.5, 0)+0.1)

# Main variables:
# pop - population
# tec - total energy consumption
# tecpc - total energy consumption per capita
pop <- read.eia("../data/Population.csv")
pop <- setNames(pop$X2010, rownames(pop)) * 1e6
tec <- read.eia("../data/Total Energy Use.csv")
tec <- setNames(tec$X2010, rownames(tec)) * 33433000.
pop <- pop[order(names(pop))]
tec <- tec[order(names(tec))]
tec <- tec[names(tec) %in% names(pop)]

not.na <- !is.na(tec) & !is.na(pop)
tec <- tec[not.na]
pop <- pop[not.na]

tecpc <- setNames(tec/pop, names(tec))
ord <- order(tecpc)
tecpc <- tecpc[ord]
pop <- pop[ord]

# Compute the CDF
ydata <- setNames(1-cumsum(pop)/sum(pop), names(pop))

# Write the CDF to file
data <- data.frame(tecpc, ydata)
colnames(data) <- c("2010 Energy consumption per capita (KiloWatts per Capita)",
		    "2010 Cumulative fraction of world population")
write.csv(data, "Figure 3a.csv", row.names=F)

# Plot the CDF
plot(tecpc, ydata,
     main="Cumulative Distribution of Energy Consumption",
     xlab="KiloWatts per Capita",
     ylab="Fraction of World Population",
     xaxt="n", yaxt="n", cex.main=1.5, font.main=1,
     pch=20, type="o", cex=0.7, cex.lab=1.6, col="red")
axis(1, at=seq(0, 30, 5), tck=-0.03)
axis(1, at=0:30, labels=F)
axis(2, at=seq(0, 1, 0.2), tck=-0.03)
axis(2, at=seq(0, 1, 0.05), labels=F)
curve(1/exp(x/2.5), add=T, lty=2, col="black", lwd=1.5)

legend("topright",
       legend=c("2010 EIA data",
		"Exponential fit"),
       lty=c(1, 2), lwd=c(1, 1.5), pch=c(20, NA), col=c("red", "black"),
       cex=1.4)

par(col="blue")
text(tecpc["United States"], ydata["United States"], "USA", adj=c(1, 1))
text(tecpc["United Arab Emirates"],
     ydata["United Arab Emirates"], "UAE", adj=c(0.5, -0.3))
text(tecpc["China"], ydata["China"], "CHN", adj=c(1.1, 1.1))
text(tecpc["Pakistan"], ydata["Pakistan"], "PAK", adj=c(-0.1, -0.1))
text(tecpc["India"], ydata["India"], "IND", adj=c(-0.1, -0.1))
text(tecpc["Brazil"], ydata["Brazil"], "BRA", adj=c(-0.1, -0.1))
text(tecpc["Russia"], ydata["Russia"], "RUS", adj=c(1.1, 1.1))
text(tecpc["France"], ydata["France"], "FRA", adj=c(-0.1, -0.1))
text(tecpc["Nigeria"], ydata["Nigeria"], "NGA", adj=c(-0.1, -0.1))
text(tecpc["United Kingdom"],
     ydata["United Kingdom"], "UK", adj=c(1.1, 1.1))
interesting <- c("United States", "United Arab Emirates",
		 "China",
		 "Pakistan",
		 "India",
		 "Brazil",
		 "Russia",
		 "France",
		 "United Kingdom",
		 "Nigeria")
points(tecpc[interesting], ydata[interesting], 
     xaxt="n", yaxt="n", cex.main=1.5, font.main=1,
     pch=20, type="p", cex=1.3, cex.lab=1.6, col="blue")

par(col="black")

temp <- sum(tec)/sum(pop)
arrows(temp, 0.05, temp, -0.025, 0.1)
text(-0.5, 0.05, "Average\n2.5 kW", adj=c(0, -0.05), cex=1.3)



# Inset

par(fig=c(0.25, 1, 0.2, 0.85), new=TRUE)
plot(tecpc, ydata, xlab=NA, ylab=NA,
     xaxt="n", yaxt="n", cex.main=1.5, font.main=1,
     pch=20, type="o", cex=0.7, cex.lab=1.6, col="red", log="y")
axis(1, at=seq(0, 30, 5), tck=-0.05)
axis(1, at=0:30, labels=F)
axis(2, at=c(1, 0.1, 0.01, 0.001), labels=c("1", "0.1", "0.01", "0.001"), tck=-0.05)
axis(2, at=c(seq(0.1, 1, 0.1), seq(0.01, 0.1, 0.01), seq(0.001, 0.01, 0.001), seq(0.0001, 0.001, 0.0001)), labels=F)
curve(1/exp(x/2.5), add=T, lty=2, col="black", lwd=1.5)

interesting <- c("United States", "United Arab Emirates",
		 "China",
		 "India",
		 "Russia",
		 "France",
		 "United Kingdom")
par(col="blue")
text(tecpc["United States"], ydata["United States"], "USA", adj=c(1, 1))
text(tecpc["United Arab Emirates"],
     ydata["United Arab Emirates"], "UAE", adj=c(-0.1, -0.3))
text(tecpc["China"], ydata["China"], "CHN", adj=c(1.1, 1.1))
text(tecpc["India"], ydata["India"], "IND", adj=c(-0.1, -0.1))
text(tecpc["Russia"], ydata["Russia"], "RUS", adj=c(1.1, 1.1))
text(tecpc["France"], ydata["France"], "FRA", adj=c(-0.1, -0.1))
text(tecpc["United Kingdom"],
     ydata["United Kingdom"], "UK", adj=c(1.1, 1.1))
points(tecpc[interesting], ydata[interesting], 
     xaxt="n", yaxt="n", cex.main=1.5, font.main=1,
     pch=20, type="p", cex=1.2, cex.lab=1.6, col="blue")
par(col="black")
